@@ -95,11 +95,25 @@ class Sixpack(object):
 
         try:
             experiment = Experiment.find(experiment_name, self.redis)
-            experiment.convert(client)
+            alternative = experiment.convert(client)
         except ValueError as e:
             return json_error({'message': str(e)}, request, 400)
 
-        return json_success({}, request)
+        resp = {
+            'alternative': {
+                'name': alternative
+            },
+            'experiment': {
+                'name': experiment.name,
+                'version': experiment.version()
+            },
+            'conversion': {
+                'value': None
+            },
+            'client_id': client_id
+        }
+
+        return json_success(resp, request)
 
     @service_unavailable_on_connection_error
     def on_participate(self, request):
@@ -129,7 +143,9 @@ class Sixpack(object):
             alternative = experiment.get_alternative(client).name
 
         resp = {
-            'alternative': alternative,
+            'alternative': {
+                'name': alternative
+            },
             'experiment': {
                 'name': experiment.name,
                 'version': experiment.version()
